@@ -174,10 +174,10 @@ set_revision (EContact *contact)
 {
 	char time_string[100] = {0};
 	const struct tm *tm = NULL;
-	GTimeVal tv;
+	time_t t;
 
-	g_get_current_time (&tv);
-	tm = gmtime (&tv.tv_sec);
+	t = time(NULL);
+	tm = gmtime (&t);
 	if (tm)
 		strftime (time_string, 100, "%Y-%m-%dT%H:%M:%SZ", tm);
 	e_contact_set (contact, E_CONTACT_REV, time_string);
@@ -551,7 +551,7 @@ book_view_thread (gpointer data)
 
 	/* ref the book view because it'll be removed and unrefed
 	   when/if it's stopped */
-	bonobo_object_ref (book_view);
+	e_data_book_view_ref (book_view);
 
 	db = bf->priv->file_db;
 	query = e_data_book_view_get_card_query (book_view);
@@ -646,7 +646,7 @@ done:
 
 	/* unref the */
 	printf("book_view file uref \n");
-	bonobo_object_unref (book_view);
+	e_data_book_view_unref (book_view);
 
 	d(printf ("finished population of book view\n"));
 
