@@ -3,7 +3,7 @@
  * Authors:
  *   Chris Toshok <toshok@ximian.com>
  *
- * Copyright (C) 2003 Ximian, Inc.
+ * Copyright (C) 1999-2008 Novell, Inc. (www.novell.com)
  */
 
 #ifndef __E_CONTACT_H__
@@ -199,6 +199,16 @@ typedef enum {
 
 	E_CONTACT_TEL, /* list of strings */
 
+	E_CONTACT_IM_SKYPE_HOME_1,     /* Synthetic string field */
+	E_CONTACT_IM_SKYPE_HOME_2,     /* Synthetic string field */
+	E_CONTACT_IM_SKYPE_HOME_3,     /* Synthetic string field */
+	E_CONTACT_IM_SKYPE_WORK_1,     /* Synthetic string field */
+	E_CONTACT_IM_SKYPE_WORK_2,     /* Synthetic string field */
+	E_CONTACT_IM_SKYPE_WORK_3,     /* Synthetic string field */
+	E_CONTACT_IM_SKYPE,    	 	/* Multi-valued */
+
+	E_CONTACT_SIP,
+
 	E_CONTACT_FIELD_LAST,
 	E_CONTACT_FIELD_FIRST        = E_CONTACT_UID,
 
@@ -336,6 +346,7 @@ const char*             e_contact_field_name       (EContactField field_id);
 const char*             e_contact_pretty_name      (EContactField field_id);
 const char*             e_contact_vcard_attribute  (EContactField field_id);
 EContactField           e_contact_field_id         (const char *field_name);
+EContactField           e_contact_field_id_from_vcard (const char *vcard_field);
 
 G_END_DECLS
 
