@@ -8,13 +8,13 @@
  * Authors:
  *   Nat Friedman <nat@ximian.com>
  *
- * Copyright 1999 - 2001, Ximian, Inc.
+ * Copyright (C) 1999-2008 Novell, Inc. (www.novell.com)
  */
 
 #include <ctype.h>
 #include <string.h>
 #include <glib.h>
- 
+
 #include "e-name-western.h"
 #include "e-name-western-tables.h"
 
@@ -28,10 +28,10 @@ typedef struct {
 } ENameWesternIdxs;
 
 static int
-e_name_western_str_count_words (char *str)
+e_name_western_str_count_words (const char *str)
 {
 	int word_count;
-	char *p;
+	const char *p;
 
 	word_count = 0;
 
@@ -105,9 +105,6 @@ e_name_western_get_words_at_idx (char *str, int idx, int num_words)
 	return g_string_free (words, FALSE);
 }
 
-/*
- * What the fuck is wrong with glib's MAX macro.
- */ 
 static int
 e_name_western_max (const int a, const int b)
 {
@@ -127,10 +124,10 @@ e_name_western_word_is_suffix (char *word)
 	 * g_utf8_casefold turns the string into lowercase, so we
 	 * don't need to casefold the suffixes.
 	 */
-	for (i = 0; e_name_western_sfx_table [i] != NULL; i ++) {
-		gboolean match = !g_utf8_collate (folded_word, e_name_western_sfx_table [i]);
+	for (i = 0; i < G_N_ELEMENTS (western_sfx_index); i++) {
+		const char *suffix = western_sfx_table + western_sfx_index[i];
 
-		if (match) {
+		if (!g_utf8_collate (folded_word, suffix)) {
 			g_free (folded_word);
 			return TRUE;
 		}
@@ -148,16 +145,18 @@ e_name_western_get_one_prefix_at_str (char *str)
 	/*
 	 * Check for prefixes from our table.
 	 */
-	for (i = 0; e_name_western_pfx_table [i] != NULL; i ++) {
+	for (i = 0; i < G_N_ELEMENTS (western_pfx_index); i++) {
 		int pfx_words;
+		const char *prefix;
 		char *words;
 		char *folded_words;
 
-		pfx_words = e_name_western_str_count_words (e_name_western_pfx_table [i]);
+		prefix = western_pfx_table + western_pfx_index[i];
+		pfx_words = e_name_western_str_count_words (prefix);
 		words = e_name_western_get_words_at_idx (str, 0, pfx_words);
 		folded_words = g_utf8_casefold (words, -1);
 
-		if (! g_utf8_collate (folded_words, e_name_western_pfx_table [i])) {
+		if (! g_utf8_collate (folded_words, prefix)) {
 			g_free (folded_words);
 			return words;
 		}
@@ -171,7 +170,7 @@ e_name_western_get_one_prefix_at_str (char *str)
 	 */
 	word = e_name_western_get_words_at_idx (str, 0, 1);
 
-	if (g_utf8_strlen (word, -1) > 2 && 
+	if (g_utf8_strlen (word, -1) > 2 &&
 	    g_unichar_isalpha (g_utf8_get_char (word)) &&
 	    g_unichar_isalpha (g_utf8_get_char (g_utf8_next_char (word))) &&
 	    word [strlen (word) - 1] == '.')
@@ -239,10 +238,9 @@ e_name_western_is_complex_last_beginning (char *word)
 	int i;
 	char *folded_word = g_utf8_casefold (word, -1);
 
-	for (i = 0; e_name_western_complex_last_table [i] != NULL; i ++) {
-
-		if (! g_utf8_collate (folded_word,
-				      e_name_western_complex_last_table [i])) {
+	for (i = 0; i < G_N_ELEMENTS (western_complex_last_index); i++) {
+		const char *last = western_complex_last_table + western_complex_last_index[i];
+		if (! g_utf8_collate (folded_word, last)) {
 			g_free (folded_word);
 			return TRUE;
 		}
@@ -317,14 +315,14 @@ e_name_western_extract_middle (ENameWestern *name, ENameWesternIdxs *idxs)
 	middle = g_utf8_next_char (middle);
 	if (*middle == '\0')
 		return;
-	
+
 	/*
 	 * Search for the first space (or the terminating \0)
 	 */
 	while (g_unichar_isspace (g_utf8_get_char (middle)) &&
 	       *middle != '\0')
 		middle = g_utf8_next_char (middle);
-		
+
 	if (*middle == '\0')
 		return;
 
@@ -337,7 +335,7 @@ e_name_western_extract_middle (ENameWestern *name, ENameWesternIdxs *idxs)
 
 		middle = name->full + idxs->nick_idx + strlen (name->nick);
 		middle = g_utf8_next_char (middle);
-		
+
 		while (g_unichar_isspace (g_utf8_get_char (middle)) &&
 		       *middle != '\0')
 			middle = g_utf8_next_char (middle);
@@ -371,7 +369,7 @@ e_name_western_extract_middle (ENameWestern *name, ENameWesternIdxs *idxs)
 		g_free (word);
 		return;
 	}
-	
+
 	idxs->middle_idx = middle - name->full;
 	name->middle = word;
 }
@@ -555,7 +553,7 @@ e_name_western_get_suffix_at_str_end (char *str)
 			break;
 		}
 		nextp = g_utf8_prev_char (nextp);
-		
+
 		e_name_western_cleanup_string (& word);
 
 		if (e_name_western_word_is_suffix (word)) {
@@ -641,7 +639,7 @@ e_name_western_reorder_asshole (ENameWestern *name, ENameWesternIdxs *idxs)
 	 * to
 	 *    <Prefix> <First name> <Middle[+nick] name> <Last name> <Suffix>
 	 */
-	
+
 	/*
 	 * Grab the prefix from the beginning.
 	 */
@@ -659,11 +657,11 @@ e_name_western_reorder_asshole (ENameWestern *name, ENameWesternIdxs *idxs)
 
 	while (g_unichar_isspace (g_utf8_get_char (p)) && *p != '\0')
 		p = g_utf8_next_char (p);
-	
-	/* 
+
+	/*
 	   Consider this case, "Br.Gate,Br. Gate,W". I know this is a damn
 	   random name, but, I got this from the bug report of 317411.
-	   
+
 	   comma = ",Br.Gate,W"
 	   prefix = "Br.Gate,Br."
 	   p = " Gate,W"
@@ -676,7 +674,7 @@ e_name_western_reorder_asshole (ENameWestern *name, ENameWesternIdxs *idxs)
 		g_free (prefix);
 		return;
 	}
-	
+
 	last = g_malloc0 (comma - p + 1);
 	strncpy (last, p, comma - p);
 
@@ -926,7 +924,7 @@ e_name_western_fixup (ENameWestern *name, ENameWesternIdxs *idxs)
 }
 
 /**
- * e_name_western_western_parse_fullname:
+ * e_name_western_parse:
  * @full_name: A string containing a western name.
  *
  * Parses @full_name and returns an #ENameWestern struct filled with
@@ -1006,7 +1004,7 @@ e_name_western_free (ENameWestern *w)
 	g_free (w->nick);
 	g_free (w->last);
 	g_free (w->suffix);
-	
+
 	g_free (w->full);
 
 	g_free (w);
