@@ -1,6 +1,6 @@
 /* Evolution calendar - iCalendar file backend
  *
- * Copyright (C) 2000-2003 Ximian, Inc.
+ * Copyright (C) 1999-2008 Novell, Inc. (www.novell.com)
  *
  * Authors: Federico Mena-Quintero <federico@ximian.com>
  *          Rodrigo Moya <rodrigo@ximian.com>
@@ -28,8 +28,6 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
-#include <bonobo/bonobo-exception.h>
-#include <bonobo/bonobo-moniker-util.h>
 #include <glib/gstdio.h>
 #include <glib/gi18n-lib.h>
 #include <gio/gio.h>
@@ -38,6 +36,7 @@
 #include <libecal/e-cal-recur.h>
 #include <libecal/e-cal-time-util.h>
 #include <libecal/e-cal-util.h>
+#include <libecal/e-cal-check-timezones.h>
 #include <libedata-cal/e-cal-backend-util.h>
 #include <libedata-cal/e-cal-backend-sexp.h>
 #include "e-cal-backend-file-events.h"
@@ -172,7 +171,7 @@ save_file_when_idle (gpointer user_data)
 		goto error;
 	}
 
-	buf = icalcomponent_as_ical_string (priv->icalcomp);
+	buf = icalcomponent_as_ical_string_r (priv->icalcomp);
 	g_output_stream_write_all (G_OUTPUT_STREAM (stream), buf, strlen (buf) * sizeof (char), NULL, NULL, &e);
 	g_free (buf);
 
@@ -739,7 +738,7 @@ notify_removals_cb (gpointer key, gpointer value, gpointer data)
 		if (!old_icomp)
 			return;
 
-		old_obj_str = icalcomponent_as_ical_string (old_icomp);
+		old_obj_str = icalcomponent_as_ical_string_r (old_icomp);
 		if (!old_obj_str)
 			return;
 
@@ -774,7 +773,7 @@ notify_adds_modifies_cb (gpointer key, gpointer value, gpointer data)
 		if (!new_icomp)
 			return;
 
-		new_obj_str = icalcomponent_as_ical_string (new_icomp);
+		new_obj_str = icalcomponent_as_ical_string_r (new_icomp);
 		if (!new_obj_str)
 			return;
 
@@ -786,8 +785,8 @@ notify_adds_modifies_cb (gpointer key, gpointer value, gpointer data)
 		if (!old_icomp || !new_icomp)
 			return;
 
-		old_obj_str = icalcomponent_as_ical_string (old_icomp);
-		new_obj_str = icalcomponent_as_ical_string (new_icomp);
+		old_obj_str = icalcomponent_as_ical_string_r (old_icomp);
+		new_obj_str = icalcomponent_as_ical_string_r (new_icomp);
 		if (!old_obj_str || !new_obj_str)
 			return;
 
@@ -908,6 +907,104 @@ get_uri_string (ECalBackend *backend)
 	return full_uri;
 }
 
+static gboolean
+add_timezone (icalcomponent *icalcomp, icaltimezone *tzone)
+{
+	GSList *to_remove = NULL, *r;
+	icalcomponent *subcomp;
+	icaltimezone *zone;
+	gboolean add = TRUE, have_same = FALSE;
+	const char *tzid;
+	char *cmp;
+
+	g_return_val_if_fail (icalcomp != NULL, FALSE);
+
+	/* it's fine to have passed in NULL tzcomp; for example UTC timezone does this */
+	if (!tzone || !icaltimezone_get_component (tzone))
+		return FALSE;
+
+	tzid = icaltimezone_get_tzid (tzone);
+	if (!tzid)
+		return FALSE;
+
+	cmp = icalcomponent_as_ical_string_r (icaltimezone_get_component (tzone));
+	zone = icaltimezone_new ();
+
+	for (subcomp = icalcomponent_get_first_component (icalcomp, ICAL_VTIMEZONE_COMPONENT);
+	     subcomp;
+	     subcomp = icalcomponent_get_next_component (icalcomp, ICAL_VTIMEZONE_COMPONENT)) {
+		if (!icaltimezone_set_component (zone, icalcomponent_new_clone (subcomp))) {
+			to_remove = g_slist_prepend (to_remove, subcomp);
+		} else if (icaltimezone_get_tzid (zone) && g_str_equal (tzid, icaltimezone_get_tzid (zone))) {
+			/* there is a timezone component with the same tzid already */
+			if (have_same) {
+				to_remove = g_slist_prepend (to_remove, subcomp);
+			} else {
+				char *str = icalcomponent_as_ical_string_r (subcomp);
+
+				/* not the best way how to compare two components, but don't have better */
+				if (str && g_str_equal (cmp, str)) {
+					have_same = TRUE;
+					add = FALSE;
+				} else {
+					to_remove = g_slist_prepend (to_remove, subcomp);
+				}
+
+				g_free (str);
+			}
+		}
+	}
+
+	g_free (cmp);
+
+	for (r = to_remove; r; r = r->next) {
+		icalcomponent_remove_component (icalcomp, r->data);
+	}
+
+	if (g_slist_length (to_remove) > 1) {
+		/* there were more than once tzid as this,
+		   thus check for duplicities for all of timezones there */
+		GHashTable *known = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
+		GSList *rem2 = NULL;
+
+		for (subcomp = icalcomponent_get_first_component (icalcomp, ICAL_VTIMEZONE_COMPONENT);
+		     subcomp;
+		     subcomp = icalcomponent_get_next_component (icalcomp, ICAL_VTIMEZONE_COMPONENT)) {
+			if (!icaltimezone_set_component (zone, icalcomponent_new_clone (subcomp))) {
+				rem2 = g_slist_prepend (rem2, subcomp);
+			} else {
+				const char *tzid2 = icaltimezone_get_tzid (zone);
+
+				/* check all but not the one which was checked above */
+				if (tzid2 && !g_str_equal (tzid, tzid2)) {
+					if (g_hash_table_lookup (known, tzid2) == NULL) {
+						/* the first component of this tzid, keep it */
+						g_hash_table_insert (known, g_strdup (tzid2), GINT_TO_POINTER (1));
+					} else {
+						/* it's there already, remove it */
+						rem2 = g_slist_prepend (rem2, subcomp);
+					}
+				}
+			}
+		}
+
+		for (r = rem2; r; r = r->next) {
+			icalcomponent_remove_component (icalcomp, r->data);
+		}
+
+		g_slist_free (rem2);
+		g_hash_table_unref (known);
+	}
+
+	icaltimezone_free (zone, TRUE);
+	g_slist_free (to_remove);
+
+	if (add)
+		icalcomponent_add_component (icalcomp, icalcomponent_new_clone (icaltimezone_get_component (tzone)));
+
+	return add || to_remove != NULL;
+}
+
 /* Open handler for the file backend */
 static ECalBackendSyncStatus
 e_cal_backend_file_open (ECalBackendSync *backend, EDataCal *cal, gboolean only_if_exists,
@@ -946,10 +1043,7 @@ e_cal_backend_file_open (ECalBackendSync *backend, EDataCal *cal, gboolean only_
 	}
 
 	if (status == GNOME_Evolution_Calendar_Success) {
-		if (priv->default_zone) {
-			icalcomponent *icalcomp = icaltimezone_get_component (priv->default_zone);
-
-			icalcomponent_add_component (priv->icalcomp, icalcomponent_new_clone (icalcomp));
+		if (priv->default_zone && add_timezone (priv->icalcomp, priv->default_zone)) {
 			save (cbfile);
 		}
 	}
@@ -1021,7 +1115,12 @@ e_cal_backend_file_remove (ECalBackendSync *backend, EDataCal *cal)
         g_free (full_path);
 
         g_static_rec_mutex_unlock (&priv->idle_save_rmutex);
-        return status;
+
+	/* lie here a bit, but otherwise the calendar will not be removed, even it should */
+	if (status != GNOME_Evolution_Calendar_Success)
+		g_print (G_STRLOC ": %s", e_cal_backend_status_to_string (status));
+
+        return GNOME_Evolution_Calendar_Success;
 }
 
 /* is_loaded handler for the file backend */
@@ -1141,7 +1240,7 @@ e_cal_backend_file_get_object (ECalBackendSync *backend, EDataCal *cal, const ch
 				return GNOME_Evolution_Calendar_ObjectNotFound;
                         }
 
-			*object = icalcomponent_as_ical_string (icalcomp);
+			*object = icalcomponent_as_ical_string_r (icalcomp);
 
 			icalcomponent_free (icalcomp);
 		}
@@ -1158,7 +1257,7 @@ e_cal_backend_file_get_object (ECalBackendSync *backend, EDataCal *cal, const ch
 			/* add all detached recurrences */
 			g_hash_table_foreach (obj_data->recurrences, (GHFunc) add_detached_recur_to_vcalendar, icalcomp);
 
-			*object = icalcomponent_as_ical_string (icalcomp);
+			*object = icalcomponent_as_ical_string_r (icalcomp);
 
 			icalcomponent_free (icalcomp);
 		} else
@@ -1205,7 +1304,7 @@ e_cal_backend_file_get_timezone (ECalBackendSync *backend, EDataCal *cal, const
 		return GNOME_Evolution_Calendar_InvalidObject;
 	}
 
-	*object = icalcomponent_as_ical_string (icalcomp);
+	*object = icalcomponent_as_ical_string_r (icalcomp);
 
 	g_static_rec_mutex_unlock (&priv->idle_save_rmutex);
 	return GNOME_Evolution_Calendar_Success;
@@ -1552,7 +1651,7 @@ e_cal_backend_file_get_free_busy (ECalBackendSync *backend, EDataCal *cal, GList
 	if (users == NULL) {
 		if (e_cal_backend_mail_account_get_default (&address, &name)) {
 			vfb = create_user_free_busy (cbfile, address, name, start, end);
-			calobj = icalcomponent_as_ical_string (vfb);
+			calobj = icalcomponent_as_ical_string_r (vfb);
 			*freebusy = g_list_append (*freebusy, calobj);
 			icalcomponent_free (vfb);
 			g_free (address);
@@ -1563,7 +1662,7 @@ e_cal_backend_file_get_free_busy (ECalBackendSync *backend, EDataCal *cal, GList
 			address = l->data;
 			if (e_cal_backend_mail_account_is_valid (address, &name)) {
 				vfb = create_user_free_busy (cbfile, address, name, start, end);
-				calobj = icalcomponent_as_ical_string (vfb);
+				calobj = icalcomponent_as_ical_string_r (vfb);
 				*freebusy = g_list_append (*freebusy, calobj);
 				icalcomponent_free (vfb);
 				g_free (name);
@@ -1853,7 +1952,7 @@ e_cal_backend_file_create_object (ECalBackendSync *backend, EDataCal *cal, char
 	e_cal_component_set_icalcomponent (comp, icalcomp);
 
 	/* Set the created and last modified times on the component */
-	current = icaltime_from_timet (time (NULL), 0);
+	current = icaltime_current_time_with_zone (icaltimezone_get_utc_timezone ());
 	e_cal_component_set_created (comp, &current);
 	e_cal_component_set_last_modified (comp, &current);
 
@@ -1923,6 +2022,7 @@ e_cal_backend_file_modify_object (ECalBackendSync *backend, EDataCal *cal, const
 	ECalComponent *comp, *recurrence;
 	ECalBackendFileObject *obj_data;
 	struct icaltimetype current;
+	GList *detached = NULL;
 
 	cbfile = E_CAL_BACKEND_FILE (backend);
 	priv = cbfile->priv;
@@ -1958,7 +2058,7 @@ e_cal_backend_file_modify_object (ECalBackendSync *backend, EDataCal *cal, const
 	e_cal_component_set_icalcomponent (comp, icalcomp);
 
 	/* Set the last modified time on the component */
-	current = icaltime_from_timet (time (NULL), 0);
+	current = icaltime_current_time_with_zone (icaltimezone_get_utc_timezone ());
 	e_cal_component_set_last_modified (comp, &current);
 
 	/* sanitize the component*/
@@ -2075,71 +2175,45 @@ e_cal_backend_file_modify_object (ECalBackendSync *backend, EDataCal *cal, const
 		rid = NULL;
 		break;
 	case CALOBJ_MOD_ALL :
-		/* in this case, we blow away all recurrences, and start over
-		   with a clean component */
-
-		if (e_cal_util_component_has_recurrences (icalcomp) && rid && *rid) {
-			icaltimetype start, recur = icaltime_from_string (rid);
-
-			start = icalcomponent_get_dtstart (icalcomp);
-
-			/* This means its a instance generated from master object. So replace
-			    the dates stored dates from the master object */
-			if (!recur.zone)
-				recur.zone = start.zone;
-
-			if (icaltime_compare_date_only (start, recur) == 0) {
-				icaltimetype end = icalcomponent_get_dtend (icalcomp);
-				ECalComponentDateTime m_sdate, m_endate;
-
-				e_cal_component_get_dtstart (obj_data->full_object, &m_sdate);
-				e_cal_component_get_dtend (obj_data->full_object, &m_endate);
-
-				if (icaltime_compare (start, recur) != 0 ||
-				    !m_endate.value ||
-				    icaltime_compare (end, *(m_endate.value)) != 0) {
-
-					m_sdate.value->hour = start.hour;
-					m_sdate.value->minute = start.minute;
-					m_sdate.value->second = start.second;
-
-					if (!m_endate.value) {
-						/* create one if not exists and make same date
-						   and time zone like start */
-						m_endate.value = g_new (struct icaltimetype, 1);
-						*m_endate.value = *m_sdate.value;
-
-						if (m_endate.tzid)
-							g_free ((char*)m_endate.tzid);
-
-						m_endate.tzid = g_strdup (m_sdate.tzid);
-					}
-
-					m_endate.value->hour = end.hour;
-					m_endate.value->minute = end.minute;
-					m_endate.value->second = end.second;
-				}
-
-				e_cal_component_set_dtstart (comp, &m_sdate);
-				e_cal_component_set_dtend (comp, &m_endate);
-				e_cal_component_set_recurid (comp, NULL);
-				e_cal_component_commit_sequence (comp);
-
-				e_cal_component_free_datetime (&m_sdate);
-				e_cal_component_free_datetime (&m_endate);
-			}
-			e_cal_component_set_recurid (comp, NULL);
-			*new_object = e_cal_component_get_as_string (comp);
-		}
+		*new_object = e_cal_component_get_as_string (comp);
 
 		/* Remove the old version */
 		if (old_object)
 			*old_object = e_cal_component_get_as_string (obj_data->full_object);
 
+		if (obj_data->recurrences_list) {
+			/* has detached components, preserve them */
+			GList *l;
+
+			for (l = obj_data->recurrences_list; l; l = l->next) {
+				detached = g_list_prepend (detached, g_object_ref (l->data));
+			}
+		}
+
 		remove_component (cbfile, comp_uid, obj_data);
 
 		/* Add the new object */
 		add_component (cbfile, comp, TRUE);
+
+		if (detached) {
+			/* it had some detached components, place them back */
+			comp_uid = icalcomponent_get_uid (e_cal_component_get_icalcomponent (comp));
+
+			if ((obj_data = g_hash_table_lookup (priv->comp_uid_hash, comp_uid)) != NULL) {
+				GList *l;
+
+				for (l = detached; l; l = l->next) {
+					ECalComponent *c = l->data;
+
+					g_hash_table_insert (obj_data->recurrences, e_cal_component_get_recurid_as_string (c), c);
+					icalcomponent_add_component (priv->icalcomp, e_cal_component_get_icalcomponent (c));
+					priv->comp = g_list_append (priv->comp, c);
+					obj_data->recurrences_list = g_list_append (obj_data->recurrences_list, c);
+				}
+			}
+
+			g_list_free (detached);
+		}
 		break;
 	}
 
@@ -2300,7 +2374,7 @@ e_cal_backend_file_remove_object (ECalBackendSync *backend, EDataCal *cal,
 }
 
 static gboolean
-cancel_received_object (ECalBackendFile *cbfile, icalcomponent *icalcomp)
+cancel_received_object (ECalBackendFile *cbfile, icalcomponent *icalcomp, char **old_object, char **new_object)
 {
 	ECalBackendFileObject *obj_data;
 	ECalBackendFilePrivate *priv;
@@ -2309,6 +2383,9 @@ cancel_received_object (ECalBackendFile *cbfile, icalcomponent *icalcomp)
 
 	priv = cbfile->priv;
 
+	*old_object = NULL;
+	*new_object = NULL;
+
 	/* Find the old version of the component. */
 	obj_data = g_hash_table_lookup (priv->comp_uid_hash, icalcomponent_get_uid (icalcomp));
 	if (!obj_data)
@@ -2321,10 +2398,14 @@ cancel_received_object (ECalBackendFile *cbfile, icalcomponent *icalcomp)
 		return FALSE;
 	}
 
+	*old_object = e_cal_component_get_as_string (obj_data->full_object);
+
+	/* new_object is kept NULL if not removing the instance */
 	rid = e_cal_component_get_recurid_as_string (comp);
-	if (rid && *rid)
+	if (rid && *rid) {
 		remove_instance (cbfile, obj_data, rid);
-	else
+		*new_object = e_cal_component_get_as_string (obj_data->full_object);
+	} else
 		remove_component (cbfile, icalcomponent_get_uid (icalcomp), obj_data);
 
 	g_free (rid);
@@ -2516,8 +2597,9 @@ e_cal_backend_file_receive_objects (ECalBackendSync *backend, EDataCal *cal, con
 	/* Now we manipulate the components we care about */
 	for (l = comps; l; l = l->next) {
 		const char *uid;
-		char *object, *old_object, *rid;
+		char *object, *old_object, *rid, *new_object;
 		ECalBackendFileObject *obj_data;
+		gboolean is_declined;
 
 		subcomp = l->data;
 
@@ -2526,7 +2608,7 @@ e_cal_backend_file_receive_objects (ECalBackendSync *backend, EDataCal *cal, con
 		e_cal_component_set_icalcomponent (comp, subcomp);
 
 		/* Set the created and last modified times on the component */
-		current = icaltime_from_timet (time (NULL), 0);
+		current = icaltime_current_time_with_zone (icaltimezone_get_utc_timezone ());
 		e_cal_component_set_created (comp, &current);
 		e_cal_component_set_last_modified (comp, &current);
 
@@ -2542,8 +2624,10 @@ e_cal_backend_file_receive_objects (ECalBackendSync *backend, EDataCal *cal, con
 		case ICAL_METHOD_PUBLISH:
 		case ICAL_METHOD_REQUEST:
 		case ICAL_METHOD_REPLY:
+			is_declined = e_cal_backend_user_declined (subcomp);
+
 			/* handle attachments */
-			if (e_cal_component_has_attachments (comp))
+			if (!is_declined && e_cal_component_has_attachments (comp))
 				fetch_attachments (backend, comp);
 			obj_data = g_hash_table_lookup (priv->comp_uid_hash, uid);
 			if (obj_data) {
@@ -2552,13 +2636,27 @@ e_cal_backend_file_receive_objects (ECalBackendSync *backend, EDataCal *cal, con
 					remove_instance (cbfile, obj_data, rid);
 				else
 					remove_component (cbfile, uid, obj_data);
-				add_component (cbfile, comp, FALSE);
+
+				if (!is_declined)
+					add_component (cbfile, comp, FALSE);
 
 				object = e_cal_component_get_as_string (comp);
-				e_cal_backend_notify_object_modified (E_CAL_BACKEND (backend), old_object, object);
+				if (!is_declined)
+					e_cal_backend_notify_object_modified (E_CAL_BACKEND (backend), old_object, object);
+				else {
+					ECalComponentId *id = e_cal_component_get_id (comp);
+
+					if (rid)
+						e_cal_backend_notify_object_removed (E_CAL_BACKEND (backend), id, old_object, object);
+					else
+						e_cal_backend_notify_object_removed (E_CAL_BACKEND (backend), id, old_object, NULL);
+
+					e_cal_component_free_id (id);
+				}
+
 				g_free (object);
 				g_free (old_object);
-			} else {
+			} else if (!is_declined) {
 				add_component (cbfile, comp, FALSE);
 
 				object = e_cal_component_get_as_string (comp);
@@ -2584,26 +2682,22 @@ e_cal_backend_file_receive_objects (ECalBackendSync *backend, EDataCal *cal, con
 			goto error;
 			break;
 		case ICAL_METHOD_CANCEL:
-			if (cancel_received_object (cbfile, subcomp)) {
+			old_object = NULL;
+			new_object = NULL;
+			if (cancel_received_object (cbfile, subcomp, &old_object, &new_object)) {
 				ECalComponentId *id;
-				object =  icalcomponent_as_ical_string (subcomp);
-				obj_data = g_hash_table_lookup (priv->comp_uid_hash, uid);
-				if (obj_data)
-					old_object = e_cal_component_get_as_string (obj_data->full_object);
-				else
-					old_object = NULL;
 
 				id = e_cal_component_get_id (comp);
 
-				e_cal_backend_notify_object_removed (E_CAL_BACKEND (backend), id, old_object, object);
+				e_cal_backend_notify_object_removed (E_CAL_BACKEND (backend), id, old_object, new_object);
 
 				/* remove the component from the toplevel VCALENDAR */
 				icalcomponent_remove_component (toplevel_comp, subcomp);
 				icalcomponent_free (subcomp);
 				e_cal_component_free_id (id);
 
+				g_free (new_object);
 				g_free (old_object);
-				g_free (object);
 			}
 			g_free (rid);
 			break;
@@ -2626,6 +2720,27 @@ e_cal_backend_file_receive_objects (ECalBackendSync *backend, EDataCal *cal, con
 
 	g_list_free (del_comps);
 
+        /* check and patch timezones */
+        {
+            GError *error = NULL;
+            if (!e_cal_check_timezones(toplevel_comp,
+                                       NULL,
+                                       e_cal_tzlookup_icomp,
+                                       priv->icalcomp,
+                                       &error)) {
+                /*
+                 * This makes assumptions about what kind of
+                 * errors can occur inside e_cal_check_timezones().
+                 * We control it, so that should be safe, but
+                 * is the code really identical with the calendar
+                 * status?
+                 */
+                status = error->code;
+                g_clear_error(&error);
+                goto error;
+            }
+        }
+
 	/* Merge the iCalendar components with our existing VCALENDAR,
 	   resolving any conflicting TZIDs. */
 	icalcomponent_merge_component (priv->icalcomp, toplevel_comp);
