@@ -1,8 +1,7 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /* Evolution calendar - generic backend class
  *
- * Copyright (C) 2000 Ximian, Inc.
- * Copyright (C) 2001 Ximian, Inc.
+ * Copyright (C) 1999-2008 Novell, Inc. (www.novell.com)
  *
  * Authors: Rodrigo Moya <rodrigo@ximian.com>
  *
@@ -34,6 +33,10 @@ G_BEGIN_DECLS
 gboolean e_cal_backend_mail_account_get_default (char **address, char **name);
 gboolean e_cal_backend_mail_account_is_valid (char *user, char **name);
 
+const char *e_cal_backend_status_to_string (GNOME_Evolution_Calendar_CallStatus status);
+
+gboolean e_cal_backend_user_declined (icalcomponent *icalcomp);
+
 G_END_DECLS
 
 #endif
