@@ -4,7 +4,7 @@
 /* 
  * Authors: Dan Winship <danw@ximian.com>
  *
- * Copyright (C) 2001 Ximian, Inc.
+ * Copyright (C) 1999-2008 Novell, Inc. (www.novell.com)
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of version 2 of the GNU Lesser General Public 
@@ -21,6 +21,7 @@
  * USA
  */
 
+#ifndef CAMEL_DISABLE_DEPRECATED
 
 #ifndef CAMEL_DISCO_STORE_H
 #define CAMEL_DISCO_STORE_H 1
@@ -104,11 +105,11 @@ typedef struct {
 CamelType camel_disco_store_get_type (void);
 
 /* Public methods */
-CamelDiscoStoreStatus camel_disco_store_status           (CamelDiscoStore *);
-void                  camel_disco_store_set_status       (CamelDiscoStore *,
-							  CamelDiscoStoreStatus,
-							  CamelException *);
-gboolean              camel_disco_store_can_work_offline (CamelDiscoStore *);
+CamelDiscoStoreStatus camel_disco_store_status           (CamelDiscoStore *store);
+void                  camel_disco_store_set_status       (CamelDiscoStore *store,
+							  CamelDiscoStoreStatus status,
+							  CamelException *ex);
+gboolean              camel_disco_store_can_work_offline (CamelDiscoStore *store);
 
 
 /* Convenience functions */
@@ -118,3 +119,5 @@ void camel_disco_store_prepare_for_offline(CamelDiscoStore *store, CamelExceptio
 G_END_DECLS
 
 #endif /* CAMEL_DISCO_STORE_H */
+
+#endif /* CAMEL_DISABLE_DEPRECATED */
