@@ -2,7 +2,7 @@
 /*
  *  Authors: Jeffrey Stedfast <fejj@ximian.com>
  *
- *  Copyright 2002 Ximian, Inc. (www.ximian.com)
+ *  Copyright (C) 1999-2008 Novell, Inc. (www.novell.com)
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -16,7 +16,7 @@
  *
  *  You should have received a copy of the GNU Lesser General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  */
 
@@ -33,40 +33,38 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <sys/time.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/time.h>
-#include <signal.h>
 #include <unistd.h>
+#include <signal.h>
 #include <fcntl.h>
 #include <errno.h>
 #include <ctype.h>
 
 #include <glib.h>
+#include <glib/gi18n-lib.h>
 #include <glib/gstdio.h>
 
 #ifndef G_OS_WIN32
 #include <sys/ioctl.h>
-#include <sys/wait.h>
 #include <sys/poll.h>
+#include <sys/wait.h>
 #include <termios.h>
 #endif
 
-#include "libedataserver/e-iconv.h"
-
-#include "camel-gpg-context.h"
-#include "camel-mime-filter-charset.h"
-#include "camel-stream-filter.h"
-#include "camel-stream-mem.h"
-#include "camel-stream-fs.h"
-#include "camel-operation.h"
-#include "camel-mime-part.h"
-#include "camel-mime-filter-canon.h"
-#include "camel-multipart-signed.h"
-#include "camel-multipart-encrypted.h"
 #include "camel-debug.h"
-#include "camel-i18n.h"
+#include "camel-gpg-context.h"
+#include "camel-iconv.h"
+#include "camel-mime-filter-canon.h"
+#include "camel-mime-filter-charset.h"
+#include "camel-mime-part.h"
+#include "camel-multipart-encrypted.h"
+#include "camel-multipart-signed.h"
+#include "camel-operation.h"
+#include "camel-stream-filter.h"
+#include "camel-stream-fs.h"
+#include "camel-stream-mem.h"
 
 #define d(x) 
 
@@ -82,7 +80,7 @@ static CamelCipherContextClass *parent_class = NULL;
  *
  * Creates a new gpg cipher context object.
  *
- * Returns a new gpg cipher context object.
+ * Returns: a new gpg cipher context object.
  **/
 CamelCipherContext *
 camel_gpg_context_new (CamelSession *session)
@@ -169,7 +167,7 @@ enum _GpgCtxMode {
 	GPG_CTX_MODE_ENCRYPT,
 	GPG_CTX_MODE_DECRYPT,
 	GPG_CTX_MODE_IMPORT,
-	GPG_CTX_MODE_EXPORT,
+	GPG_CTX_MODE_EXPORT
 };
 
 enum _GpgTrustMetric {
@@ -299,7 +297,7 @@ gpg_ctx_new (CamelSession *session)
 	gpg->diagbuf = CAMEL_STREAM_MEM (stream)->buffer;
 	gpg->diagflushed = FALSE;
 	
-	if ((charset = e_iconv_locale_charset ()) && g_ascii_strcasecmp (charset, "UTF-8") != 0) {
+	if ((charset = camel_iconv_locale_charset ()) && g_ascii_strcasecmp (charset, "UTF-8") != 0) {
 		CamelMimeFilterCharset *filter;
 		CamelStreamFilter *fstream;
 		
@@ -401,7 +399,7 @@ gpg_ctx_get_diagnostics (struct _GpgCtx *gpg)
 		if (gpg->diagbuf->len == 0)
 			return NULL;
 		
-		g_byte_array_append (gpg->diagbuf, "", 1);
+		g_byte_array_append (gpg->diagbuf, (guchar *) "", 1);
 	}
 	
 	return (const char *) gpg->diagbuf->data;
@@ -504,6 +502,7 @@ gpg_ctx_get_argv (struct _GpgCtx *gpg, int status_fd, char **sfd, int passwd_fd,
 	g_ptr_array_add (argv, "--no-secmem-warning");
 	g_ptr_array_add (argv, "--no-greeting");
 	g_ptr_array_add (argv, "--no-tty");
+	
 	if (passwd_fd == -1) {
 		/* only use batch mode if we don't intend on using the
                    interactive --command-fd option */
@@ -656,6 +655,7 @@ gpg_ctx_op_start (struct _GpgCtx *gpg)
 	close (fds[5]);
 	gpg->status_fd = fds[6];
 	close (fds[7]);
+	
 	if (gpg->need_passwd) {
 		close (fds[8]);
 		gpg->passwd_fd = fds[9];
@@ -744,24 +744,27 @@ gpg_ctx_parse_status (struct _GpgCtx *gpg, CamelException *ex)
 	
 	*inptr++ = '\0';
 	status = gpg->statusbuf;
-
+	
 	if (camel_debug("gpg:status"))
 		printf ("status: %s\n", status);
 	
-	if (strncmp (status, "[GNUPG:] ", 9) != 0) {
+	if (strncmp ((const char *) status, "[GNUPG:] ", 9) != 0) {
+		char *message;
+		message = g_locale_to_utf8((const gchar *) status, -1, NULL, NULL, NULL);
 		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
 				      _("Unexpected GnuPG status message encountered:\n\n%s"),
-				      status);
+				      message);
+		g_free(message);
 		return -1;
 	}
 	
 	status += 9;
 	
-	if (!strncmp (status, "USERID_HINT ", 12)) {
+	if (!strncmp ((char *) status, "USERID_HINT ", 12)) {
 		char *hint, *user;
 		
 		status += 12;
-		status = next_token (status, &hint);
+		status = (const unsigned char *) next_token ((char *) status, &hint);
 		if (!hint) {
 			camel_exception_set (ex, CAMEL_EXCEPTION_SYSTEM,
 					     _("Failed to parse gpg userid hint."));
@@ -774,18 +777,18 @@ gpg_ctx_parse_status (struct _GpgCtx *gpg, CamelException *ex)
 			goto recycle;
 		}
 		
-		if (gpg->utf8 || !(user = g_locale_to_utf8 (status, -1, &nread, &nwritten, NULL)))
-			user = g_strdup (status);
+		if (gpg->utf8 || !(user = g_locale_to_utf8 ((gchar *) status, -1, &nread, &nwritten, NULL)))
+			user = g_strdup ((gchar *) status);
 		
 		g_strstrip (user);
 		
 		g_hash_table_insert (gpg->userid_hint, hint, user);
-	} else if (!strncmp (status, "NEED_PASSPHRASE ", 16)) {
+	} else if (!strncmp ((char *) status, "NEED_PASSPHRASE ", 16)) {
 		char *userid;
 		
 		status += 16;
 		
-		status = next_token (status, &userid);
+		status = (const unsigned char *) next_token ((char *) status, &userid);
 		if (!userid) {
 			camel_exception_set (ex, CAMEL_EXCEPTION_SYSTEM,
 					     _("Failed to parse gpg passphrase request."));
@@ -794,12 +797,12 @@ gpg_ctx_parse_status (struct _GpgCtx *gpg, CamelException *ex)
 		
 		g_free (gpg->need_id);
 		gpg->need_id = userid;
-	} else if (!strncmp (status, "NEED_PASSPHRASE_PIN ", 20)) {
+	} else if (!strncmp ((char *) status, "NEED_PASSPHRASE_PIN ", 20)) {
 		char *userid;
 		
 		status += 20;
 		
-		status = next_token (status, &userid);
+		status = (const unsigned char *) next_token ((char *) status, &userid);
 		if (!userid) {
 			camel_exception_set (ex, CAMEL_EXCEPTION_SYSTEM,
 					     _("Failed to parse gpg passphrase request."));
@@ -808,7 +811,7 @@ gpg_ctx_parse_status (struct _GpgCtx *gpg, CamelException *ex)
 		
 		g_free (gpg->need_id);
 		gpg->need_id = userid;
-	} else if (!strncmp (status, "GET_HIDDEN ", 11)) {
+	} else if (!strncmp ((char *) status, "GET_HIDDEN ", 11)) {
 		const char *name = NULL;
 		char *prompt, *passwd;
 		guint32 flags;
@@ -820,16 +823,18 @@ gpg_ctx_parse_status (struct _GpgCtx *gpg, CamelException *ex)
 		else if (!name)
 			name = "";
 		
-		if (!strncmp (status, "passphrase.pin.ask", 18)) {
-			prompt = g_strdup_printf (_("You need a PIN to unlock the key for your\n"
-						    "SmartCard: \"%s\""), name);
-		} else if (!strncmp (status, "passphrase.enter", 16)) {
-			prompt = g_strdup_printf (_("You need a passphrase to unlock the key for\n"
-						    "user: \"%s\""), name);
+		if (!strncmp ((char *) status, "passphrase.pin.ask", 18)) {
+			prompt = g_markup_printf_escaped (
+				_("You need a PIN to unlock the key for your\n"
+				  "SmartCard: \"%s\""), name);
+		} else if (!strncmp ((char *) status, "passphrase.enter", 16)) {
+			prompt = g_markup_printf_escaped (
+				_("You need a passphrase to unlock the key for\n"
+				  "user: \"%s\""), name);
 		} else {
-			next_token (status, &prompt);
+			next_token ((char *) status, &prompt);
 			camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
-					      _("Unexpected request from GnuPG for `%s'"), prompt);
+					      _("Unexpected request from GnuPG for '%s'"), prompt);
 			g_free (prompt);
 			return -1;
 		}
@@ -859,9 +864,9 @@ gpg_ctx_parse_status (struct _GpgCtx *gpg, CamelException *ex)
 		}
 		
 		g_free (prompt);
-	} else if (!strncmp (status, "GOOD_PASSPHRASE", 15)) {
+	} else if (!strncmp ((char *) status, "GOOD_PASSPHRASE", 15)) {
 		gpg->bad_passwds = 0;
-	} else if (!strncmp (status, "BAD_PASSPHRASE", 14)) {
+	} else if (!strncmp ((char *) status, "BAD_PASSPHRASE", 14)) {
 		gpg->bad_passwds++;
 		
 		camel_session_forget_password (gpg->session, NULL, NULL, gpg->need_id, ex);
@@ -871,13 +876,16 @@ gpg_ctx_parse_status (struct _GpgCtx *gpg, CamelException *ex)
 					     _("Failed to unlock secret key: 3 bad passphrases given."));
 			return -1;
 		}
-	} else if (!strncmp (status, "UNEXPECTED ", 11)) {
+	} else if (!strncmp ((const char *) status, "UNEXPECTED ", 11)) {
 		/* this is an error */
+		char *message;
+		message = g_locale_to_utf8((const gchar *) status+11, -1, NULL, NULL, NULL);
 		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
 				      _("Unexpected response from GnuPG: %s"),
-				      status + 11);
+				      message);
+		g_free(message);
 		return -1;
-	} else if (!strncmp (status, "NODATA", 6)) {
+	} else if (!strncmp ((char *) status, "NODATA", 6)) {
 		/* this is an error */
 		/* But we ignore it anyway, we should get other response codes to say why */
 		gpg->nodata = TRUE;
@@ -885,58 +893,61 @@ gpg_ctx_parse_status (struct _GpgCtx *gpg, CamelException *ex)
 		/* check to see if we are complete */
 		switch (gpg->mode) {
 		case GPG_CTX_MODE_SIGN:
-			if (!strncmp (status, "SIG_CREATED ", 12)) {
+			if (!strncmp ((char *) status, "SIG_CREATED ", 12)) {
 				/* FIXME: save this state? */
 			}
 			break;
+		case GPG_CTX_MODE_DECRYPT:
+			if (!strncmp ((char *) status, "BEGIN_DECRYPTION", 16)) {
+				/* nothing to do... but we know to expect data on stdout soon */
+				break;
+			} else if (!strncmp ((char *) status, "END_DECRYPTION", 14)) {
+				/* nothing to do, but we know the end is near? */
+				break;
+			}
+			/* let if fall through to verify possible signatures too */
+			/* break; */
 		case GPG_CTX_MODE_VERIFY:
-			if (!strncmp (status, "TRUST_", 6)) {
+			if (!strncmp ((char *) status, "TRUST_", 6)) {
 				status += 6;
-				if (!strncmp (status, "NEVER", 5)) {
+				if (!strncmp ((char *) status, "NEVER", 5)) {
 					gpg->trust = GPG_TRUST_NEVER;
-				} else if (!strncmp (status, "MARGINAL", 8)) {
+				} else if (!strncmp ((char *) status, "MARGINAL", 8)) {
 					gpg->trust = GPG_TRUST_MARGINAL;
-				} else if (!strncmp (status, "FULLY", 5)) {
+				} else if (!strncmp ((char *) status, "FULLY", 5)) {
 					gpg->trust = GPG_TRUST_FULLY;
-				} else if (!strncmp (status, "ULTIMATE", 8)) {
+				} else if (!strncmp ((char *) status, "ULTIMATE", 8)) {
 					gpg->trust = GPG_TRUST_ULTIMATE;
-				} else if (!strncmp (status, "UNDEFINED", 9)) {
+				} else if (!strncmp ((char *) status, "UNDEFINED", 9)) {
 					gpg->trust = GPG_TRUST_UNDEFINED;
 				}
-			} else if (!strncmp (status, "GOODSIG ", 8)) {
+			} else if (!strncmp ((char *) status, "GOODSIG ", 8)) {
 				gpg->goodsig = TRUE;
 				gpg->hadsig = TRUE;
-			} else if (!strncmp (status, "VALIDSIG ", 9)) {
+			} else if (!strncmp ((char *) status, "VALIDSIG ", 9)) {
 				gpg->validsig = TRUE;
-			} else if (!strncmp (status, "BADSIG ", 7)) {
+			} else if (!strncmp ((char *) status, "BADSIG ", 7)) {
 				gpg->badsig = FALSE;
 				gpg->hadsig = TRUE;
-			} else if (!strncmp (status, "ERRSIG ", 7)) {
+			} else if (!strncmp ((char *) status, "ERRSIG ", 7)) {
 				/* Note: NO_PUBKEY often comes after an ERRSIG */
 				gpg->errsig = FALSE;
 				gpg->hadsig = TRUE;
-			} else if (!strncmp (status, "NO_PUBKEY ", 10)) {
+			} else if (!strncmp ((char *) status, "NO_PUBKEY ", 10)) {
 				gpg->nopubkey = TRUE;
 			}
 			break;
 		case GPG_CTX_MODE_ENCRYPT:
-			if (!strncmp (status, "BEGIN_ENCRYPTION", 16)) {
+			if (!strncmp ((char *) status, "BEGIN_ENCRYPTION", 16)) {
 				/* nothing to do... but we know to expect data on stdout soon */
-			} else if (!strncmp (status, "END_ENCRYPTION", 14)) {
+			} else if (!strncmp ((char *) status, "END_ENCRYPTION", 14)) {
 				/* nothing to do, but we know the end is near? */
-			} else if (!strncmp (status, "NO_RECP", 7)) {
+			} else if (!strncmp ((char *) status, "NO_RECP", 7)) {
 				camel_exception_set (ex, CAMEL_EXCEPTION_SYSTEM,
 						     _("Failed to encrypt: No valid recipients specified."));
 				return -1;
 			}
 			break;
-		case GPG_CTX_MODE_DECRYPT:
-			if (!strncmp (status, "BEGIN_DECRYPTION", 16)) {
-				/* nothing to do... but we know to expect data on stdout soon */
-			} else if (!strncmp (status, "END_DECRYPTION", 14)) {
-				/* nothing to do, but we know the end is near? */
-			}
-			break;
 		case GPG_CTX_MODE_IMPORT:
 			/* noop */
 			break;
@@ -999,6 +1010,7 @@ gpg_ctx_op_cancel (struct _GpgCtx *gpg)
 	kill (gpg->pid, SIGTERM);
 	sleep (1);
 	retval = waitpid (gpg->pid, &status, WNOHANG);
+	
 	if (retval == (pid_t) 0) {
 		/* no more mr nice guy... */
 		kill (gpg->pid, SIGKILL);
@@ -1015,25 +1027,27 @@ gpg_ctx_op_step (struct _GpgCtx *gpg, CamelException *ex)
 #ifndef G_OS_WIN32
 	struct pollfd polls[6];
 	int status, i, cancel_fd;
-
-	for (i=0;i<6;i++)
+	
+	for (i=0;i<6;i++) {
 		polls[i].fd = -1;
-
+		polls[i].events = 0;
+	}
+	
 	if (!gpg->seen_eof1) {
 		polls[0].fd = gpg->stdout_fd;
 		polls[0].events = POLLIN;
 	}
-
+	
 	if (!gpg->seen_eof2) {
 		polls[1].fd = gpg->stderr_fd;
 		polls[1].events = POLLIN;
 	}
-
+	
 	if (!gpg->complete) {
 		polls[2].fd = gpg->status_fd;
 		polls[2].events = POLLIN;
 	}
-
+	
 	polls[3].fd = gpg->stdin_fd;
 	polls[3].events = POLLOUT;
 	polls[4].fd = gpg->passwd_fd;
@@ -1041,24 +1055,24 @@ gpg_ctx_op_step (struct _GpgCtx *gpg, CamelException *ex)
 	cancel_fd = camel_operation_cancel_fd(NULL);
 	polls[5].fd = cancel_fd;
 	polls[5].events = POLLIN;
-
+	
 	do {
 		for (i=0;i<6;i++)
 			polls[i].revents = 0;
 		status = poll(polls, 6, 30*1000);
 	} while (status == -1 && errno == EINTR);
-
+	
 	if (status == 0)
 		return 0;	/* timed out */
 	else if (status == -1)
 		goto exception;
-
+	
 	if ((polls[5].revents & POLLIN) && camel_operation_cancel_check(NULL)) {
 		camel_exception_set(ex, CAMEL_EXCEPTION_USER_CANCEL, _("Canceled."));
 		gpg_ctx_op_cancel(gpg);
 		return -1;
 	}
-
+	
 	/* Test each and every file descriptor to see if it's 'ready',
 	   and if so - do what we can with it and then drop through to
 	   the next file descriptor and so on until we've done what we
@@ -1291,12 +1305,12 @@ gpg_sign (CamelCipherContext *context, const char *userid, CamelCipherHash hash,
 				     _("Could not generate signing data: %s"), g_strerror(errno));
 		goto fail;
 	}
-
+	
 #ifdef GPG_LOG
 	if (camel_debug_start("gpg:sign")) {
 		char *name;
 		CamelStream *out;
-
+		
 		name = g_strdup_printf("camel-gpg.%d.sign-data", logid++);
 		out = camel_stream_fs_new_with_name(name, O_CREAT|O_TRUNC|O_WRONLY, 0666);
 		if (out) {
@@ -1338,24 +1352,24 @@ gpg_sign (CamelCipherContext *context, const char *userid, CamelCipherHash hash,
 				     _("Failed to execute gpg."));
 		goto fail;
 	}
-
+	
 	res = 0;
-
+	
 	dw = camel_data_wrapper_new();
 	camel_stream_reset(ostream);
 	camel_data_wrapper_construct_from_stream(dw, ostream);
-
+	
 	sigpart = camel_mime_part_new();
 	ct = camel_content_type_new("application", "pgp-signature");
 	camel_content_type_set_param(ct, "name", "signature.asc");
 	camel_data_wrapper_set_mime_type_field(dw, ct);
 	camel_content_type_unref(ct);
-
+	
 	camel_medium_set_content_object((CamelMedium *)sigpart, dw);
 	camel_object_unref(dw);
-
+	
 	camel_mime_part_set_description(sigpart, _("This is a digitally signed message part"));
-
+	
 	mps = camel_multipart_signed_new();
 	ct = camel_content_type_new("multipart", "signed");
 	camel_content_type_set_param(ct, "micalg", camel_cipher_hash_to_id(context, hash));
@@ -1363,12 +1377,12 @@ gpg_sign (CamelCipherContext *context, const char *userid, CamelCipherHash hash,
 	camel_data_wrapper_set_mime_type_field((CamelDataWrapper *)mps, ct);
 	camel_content_type_unref(ct);
 	camel_multipart_set_boundary((CamelMultipart *)mps, NULL);
-
+	
 	mps->signature = sigpart;
 	mps->contentraw = istream;
 	camel_stream_reset(istream);
 	camel_object_ref(istream);
-
+	
 	camel_medium_set_content_object((CamelMedium *)opart, (CamelDataWrapper *)mps);
 fail:
 	camel_object_unref(ostream);
@@ -1423,8 +1437,10 @@ gpg_verify (CamelCipherContext *context, CamelMimePart *ipart, CamelException *e
 	char *sigfile = NULL;
 	CamelContentType *ct;
 	CamelMimePart *sigpart;
-	CamelStream *istream = NULL;
+	CamelStream *istream = NULL, *canon_stream;
 	CamelMultipart *mps;
+	CamelStreamFilter *filter;
+	CamelMimeFilter *canon;
 
 	mps = (CamelMultipart *)camel_medium_get_content_object((CamelMedium *)ipart);
 	ct = ((CamelDataWrapper *)mps)->mime_type;
@@ -1433,7 +1449,7 @@ gpg_verify (CamelCipherContext *context, CamelMimePart *ipart, CamelException *e
 	if (camel_content_type_is(ct, "multipart", "signed")) {
 		/* PGP/Mime Signature */
 		const char *tmp;
-
+		
 		tmp = camel_content_type_param(ct, "protocol");
 		if (!CAMEL_IS_MULTIPART_SIGNED(mps)
 		    || tmp == NULL  
@@ -1442,20 +1458,19 @@ gpg_verify (CamelCipherContext *context, CamelMimePart *ipart, CamelException *e
 				      _("Cannot verify message signature: Incorrect message format"));
 			return NULL;
 		}
-	
+		
 		if (!(istream = camel_multipart_signed_get_content_stream ((CamelMultipartSigned *) mps, NULL))) {
 			camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
 				      _("Cannot verify message signature: Incorrect message format"));
 			return NULL;
 		}
-	
+		
 		if (!(sigpart = camel_multipart_get_part (mps, CAMEL_MULTIPART_SIGNED_SIGNATURE))) {
 			camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
 				      _("Cannot verify message signature: Incorrect message format"));
 			camel_object_unref (istream);
 			return NULL;
 		}
-
 	} else if (camel_content_type_is(ct, "application", "x-inlinepgp-signed")) {
 		/* Inline Signed */
 		CamelDataWrapper *content;
@@ -1476,7 +1491,7 @@ gpg_verify (CamelCipherContext *context, CamelMimePart *ipart, CamelException *e
 	if (camel_debug_start("gpg:sign")) {
 		char *name;
 		CamelStream *out;
-
+		
 		name = g_strdup_printf("camel-gpg.%d.verify.data", logid);
 		out = camel_stream_fs_new_with_name(name, O_CREAT|O_TRUNC|O_WRONLY, 0666);
 		if (out) {
@@ -1485,7 +1500,9 @@ gpg_verify (CamelCipherContext *context, CamelMimePart *ipart, CamelException *e
 			camel_stream_reset(istream);
 			camel_object_unref(out);
 		}
+		
 		g_free(name);
+		
 		if (sigpart) {
 			name = g_strdup_printf("camel-gpg.%d.verify.signature", logid++);
 			out = camel_stream_fs_new_with_name(name, O_CREAT|O_TRUNC|O_WRONLY, 0666);
@@ -1499,7 +1516,7 @@ gpg_verify (CamelCipherContext *context, CamelMimePart *ipart, CamelException *e
 		camel_debug_end();
 	}
 #endif
-
+	
 	if (sigpart) {
 		sigfile = swrite (sigpart);
 		if (sigfile == NULL) {
@@ -1511,11 +1528,26 @@ gpg_verify (CamelCipherContext *context, CamelMimePart *ipart, CamelException *e
 	}
 	
 	camel_stream_reset(istream);
+	canon_stream = camel_stream_mem_new ();
+
+	/* strip trailing white-spaces */
+	filter = camel_stream_filter_new_with_stream (canon_stream);
+	canon = camel_mime_filter_canon_new (CAMEL_MIME_FILTER_CANON_CRLF | CAMEL_MIME_FILTER_CANON_STRIP);
+	camel_stream_filter_add (filter, canon);
+	camel_object_unref (canon);
+
+	camel_stream_write_to_stream (istream, (CamelStream *)filter);
+
+	camel_object_unref (filter);
+	camel_stream_reset (istream);
+
+	camel_stream_reset (canon_stream);
+
 	gpg = gpg_ctx_new (context->session);
 	gpg_ctx_set_mode (gpg, GPG_CTX_MODE_VERIFY);
 	if (sigfile)
                 gpg_ctx_set_sigfile (gpg, sigfile);
-	gpg_ctx_set_istream (gpg, istream);
+	gpg_ctx_set_istream (gpg, canon_stream);
 	
 	if (gpg_ctx_op_start (gpg) == -1) {
 		camel_exception_set (ex, CAMEL_EXCEPTION_SYSTEM,
@@ -1539,6 +1571,8 @@ gpg_verify (CamelCipherContext *context, CamelMimePart *ipart, CamelException *e
 			validity->sign.status = CAMEL_CIPHER_VALIDITY_SIGN_GOOD;
 		else
 			validity->sign.status = CAMEL_CIPHER_VALIDITY_SIGN_BAD;
+	} else if (gpg->nopubkey) {
+		validity->sign.status = CAMEL_CIPHER_VALIDITY_SIGN_NEED_PUBLIC_KEY;
 	} else {
 		validity->sign.status = CAMEL_CIPHER_VALIDITY_SIGN_BAD;
 	}
@@ -1550,6 +1584,7 @@ gpg_verify (CamelCipherContext *context, CamelMimePart *ipart, CamelException *e
 		g_free (sigfile);
 	}
 	camel_object_unref(istream);
+	camel_object_unref (canon_stream);
 	
 	return validity;
 	
@@ -1580,7 +1615,7 @@ gpg_encrypt (CamelCipherContext *context, const char *userid, GPtrArray *recipie
 	CamelDataWrapper *dw;
 	CamelContentType *ct;
 	CamelMultipartEncrypted *mpe;
-
+	
 	ostream = camel_stream_mem_new();
 	istream = camel_stream_mem_new();
 	if (camel_cipher_canonical_to_stream(ipart, CAMEL_MIME_FILTER_CANON_CRLF, istream) == -1) {
@@ -1588,7 +1623,7 @@ gpg_encrypt (CamelCipherContext *context, const char *userid, GPtrArray *recipie
 				     _("Could not generate encrypting data: %s"), g_strerror(errno));
 		goto fail1;
 	}
-
+	
 	gpg = gpg_ctx_new (context->session);
 	gpg_ctx_set_mode (gpg, GPG_CTX_MODE_ENCRYPT);
 	gpg_ctx_set_armor (gpg, TRUE);
@@ -1596,7 +1631,7 @@ gpg_encrypt (CamelCipherContext *context, const char *userid, GPtrArray *recipie
 	gpg_ctx_set_istream (gpg, istream);
 	gpg_ctx_set_ostream (gpg, ostream);
 	gpg_ctx_set_always_trust (gpg, ctx->always_trust);
-
+	
 	for (i = 0; i < recipients->len; i++) {
 		gpg_ctx_add_recipient (gpg, recipients->pdata[i]);
 	}
@@ -1605,7 +1640,7 @@ gpg_encrypt (CamelCipherContext *context, const char *userid, GPtrArray *recipie
 		camel_exception_set (ex, CAMEL_EXCEPTION_SYSTEM, _("Failed to execute gpg."));
 		goto fail;
 	}
-
+	
 	/* FIXME: move tihs to a common routine */
 	while (!gpg_ctx_op_complete(gpg)) {
 		if (gpg_ctx_op_step (gpg, ex) == -1)
@@ -1620,27 +1655,27 @@ gpg_encrypt (CamelCipherContext *context, const char *userid, GPtrArray *recipie
 				     diagnostics && *diagnostics ? diagnostics : _("Failed to execute gpg."));
 		goto fail;
 	}
-
+	
 	res = 0;
-
+	
 	dw = camel_data_wrapper_new();
 	camel_data_wrapper_construct_from_stream(dw, ostream);
-
+	
 	encpart = camel_mime_part_new();
 	ct = camel_content_type_new("application", "octet-stream");
 	camel_content_type_set_param(ct, "name", "encrypted.asc");
 	camel_data_wrapper_set_mime_type_field(dw, ct);
 	camel_content_type_unref(ct);
-
+	
 	camel_medium_set_content_object((CamelMedium *)encpart, dw);
 	camel_object_unref(dw);
-
+	
 	camel_mime_part_set_description(encpart, _("This is a digitally encrypted message part"));
-
+	
 	vstream = camel_stream_mem_new();
 	camel_stream_write(vstream, "Version: 1\n", strlen("Version: 1\n"));
 	camel_stream_reset(vstream);
-
+	
 	verpart = camel_mime_part_new();
 	dw = camel_data_wrapper_new();
 	camel_data_wrapper_set_mime_type(dw, context->encrypt_protocol);
@@ -1648,22 +1683,22 @@ gpg_encrypt (CamelCipherContext *context, const char *userid, GPtrArray *recipie
 	camel_object_unref(vstream);
 	camel_medium_set_content_object((CamelMedium *)verpart, dw);
 	camel_object_unref(dw);
-
+	
 	mpe = camel_multipart_encrypted_new();
 	ct = camel_content_type_new("multipart", "encrypted");
 	camel_content_type_set_param(ct, "protocol", context->encrypt_protocol);
 	camel_data_wrapper_set_mime_type_field((CamelDataWrapper *)mpe, ct);
 	camel_content_type_unref(ct);
 	camel_multipart_set_boundary((CamelMultipart *)mpe, NULL);
-
+	
 	mpe->decrypted = ipart;
 	camel_object_ref(ipart);
-
+	
 	camel_multipart_add_part((CamelMultipart *)mpe, verpart);
 	camel_object_unref(verpart);
 	camel_multipart_add_part((CamelMultipart *)mpe, encpart);
 	camel_object_unref(encpart);
-
+	
 	camel_medium_set_content_object((CamelMedium *)opart, (CamelDataWrapper *)mpe);
 fail:
 	gpg_ctx_free(gpg);
@@ -1686,7 +1721,20 @@ gpg_decrypt(CamelCipherContext *context, CamelMimePart *ipart, CamelMimePart *op
 	CamelContentType *ct;
 	int rv;
 	
+	if (!ipart) {
+		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
+				_("Cannot decrypt message: Incorrect message format"));
+		return NULL;
+	}
+	
 	content = camel_medium_get_content_object((CamelMedium *)ipart);
+	
+	if (!content) {
+		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
+				_("Cannot decrypt message: Incorrect message format"));
+		return NULL;
+	}
+	
 	ct = camel_mime_part_get_content_type((CamelMimePart *)content);
 	/* Encrypted part (using our fake mime type) or PGP/Mime multipart */
 	if (camel_content_type_is(ct, "multipart", "encrypted")) {	
@@ -1695,11 +1743,10 @@ gpg_decrypt(CamelCipherContext *context, CamelMimePart *ipart, CamelMimePart *op
 			camel_exception_set (ex, CAMEL_EXCEPTION_SYSTEM, _("Failed to decrypt MIME part: protocol error"));
 			return NULL;
 		}
-
+		
 		content = camel_medium_get_content_object ((CamelMedium *) encrypted);
 	} else if (camel_content_type_is(ct, "application", "x-inlinepgp-encrypted")) {
 		content = camel_medium_get_content_object ((CamelMedium *) ipart);
-
 	} else {
 		/* Invalid Mimetype */
 		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
@@ -1713,7 +1760,7 @@ gpg_decrypt(CamelCipherContext *context, CamelMimePart *ipart, CamelMimePart *op
 	
 	ostream = camel_stream_mem_new();
 	camel_stream_mem_set_secure((CamelStreamMem *)ostream);
-
+	
 	gpg = gpg_ctx_new (context->session);
 	gpg_ctx_set_mode (gpg, GPG_CTX_MODE_DECRYPT);
 	gpg_ctx_set_istream (gpg, istream);
@@ -1755,6 +1802,7 @@ gpg_decrypt(CamelCipherContext *context, CamelMimePart *ipart, CamelMimePart *op
 		/* Set mime/type of this new part to application/octet-stream to force type snooping */
 		camel_mime_part_set_content_type(opart, "application/octet-stream");
 	}
+	
 	if (rv != -1) {	
 		valid = camel_cipher_validity_new();
 		valid->encrypt.description = g_strdup(_("Encrypted content"));
@@ -1769,7 +1817,7 @@ gpg_decrypt(CamelCipherContext *context, CamelMimePart *ipart, CamelMimePart *op
 				else
 					valid->sign.status = CAMEL_CIPHER_VALIDITY_SIGN_BAD;
 			} else if (gpg->nopubkey) {
-				valid->sign.status = CAMEL_CIPHER_VALIDITY_SIGN_UNKNOWN;
+				valid->sign.status = CAMEL_CIPHER_VALIDITY_SIGN_NEED_PUBLIC_KEY;
 			} else {
 				valid->sign.status = CAMEL_CIPHER_VALIDITY_SIGN_BAD;
 			}
@@ -1783,7 +1831,7 @@ gpg_decrypt(CamelCipherContext *context, CamelMimePart *ipart, CamelMimePart *op
 	camel_object_unref(ostream);
 	camel_object_unref(istream);
 	gpg_ctx_free (gpg);
-
+	
 	return valid;
 }
 
@@ -1792,7 +1840,7 @@ gpg_import_keys (CamelCipherContext *context, CamelStream *istream, CamelExcepti
 {
 	struct _GpgCtx *gpg;
 	int res = -1;
-
+	
 	gpg = gpg_ctx_new (context->session);
 	gpg_ctx_set_mode (gpg, GPG_CTX_MODE_IMPORT);
 	gpg_ctx_set_istream (gpg, istream);
@@ -1818,7 +1866,7 @@ gpg_import_keys (CamelCipherContext *context, CamelStream *istream, CamelExcepti
 				     _("Failed to execute gpg."));
 		goto fail;
 	}
-
+	
 	res = 0;
 fail:
 	gpg_ctx_free (gpg);
@@ -1832,7 +1880,7 @@ gpg_export_keys (CamelCipherContext *context, GPtrArray *keys, CamelStream *ostr
 	struct _GpgCtx *gpg;
 	int i;
 	int res = -1;
-
+	
 	gpg = gpg_ctx_new (context->session);
 	gpg_ctx_set_mode (gpg, GPG_CTX_MODE_EXPORT);
 	gpg_ctx_set_armor (gpg, TRUE);
@@ -1926,5 +1974,3 @@ camel_gpg_context_get_type (void)
 	
 	return type;
 }
-
-
