@@ -2,7 +2,7 @@
 /*
  *  Authors: Jeffrey Stedfast <fejj@ximian.com>
  *
- *  Copyright 2002 Ximian, Inc. (www.ximian.com)
+ *  Copyright (C) 1999-2008 Novell, Inc. (www.novell.com)
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -372,7 +372,7 @@ http_method_invoke (CamelHttpStream *http)
 		 http->url->host));
 	if (camel_stream_printf (http->raw, "%s %s HTTP/1.0\r\nUser-Agent: %s\r\nHost: %s\r\n",
 				 method,
-				 http->proxy ? url : http->url->path,
+				 url,
 				 http->user_agent ? http->user_agent : "CamelHttpStream/1.0",
 				 http->url->host) == -1) {
 		http_disconnect(http);
@@ -425,14 +425,20 @@ stream_read (CamelStream *stream, char *buffer, size_t n)
 		if (http_connect (http, http->proxy ? http->proxy : http->url) == NULL)
 			return -1;
 		
-		if (http_method_invoke (http) == -1)
+		if (http_method_invoke (http) == -1) {
+			http_disconnect(http);
 			return -1;
+		}
 		
-		if (http_get_statuscode (http) == -1)
+		if (http_get_statuscode (http) == -1) {
+			http_disconnect(http);
 			return -1;
+		}
 		
-		if (http_get_headers (http) == -1)
+		if (http_get_headers (http) == -1) {
+			http_disconnect(http);
 			return -1;
+		}
 		
 		switch (http->statuscode) {
 		case 200:
@@ -481,6 +487,9 @@ stream_read (CamelStream *stream, char *buffer, size_t n)
 		}
 	}
 	
+	if (n == 0)
+		return 0;
+	
 	nread = camel_mime_parser_read (http->parser, &parser_buf, n);
 	
 	if (nread > 0)
@@ -540,13 +549,7 @@ camel_http_stream_get_content_type (CamelHttpStream *http_stream)
 	g_return_val_if_fail (CAMEL_IS_HTTP_STREAM (http_stream), NULL);
 
 	if (!http_stream->content_type && !http_stream->raw) {
-		if (http_connect (http_stream, http_stream->proxy ? http_stream->proxy : http_stream->url) == NULL)
-			return NULL;
-
-		if (http_method_invoke (http_stream) == -1)
-			return NULL;
-
-		if (http_get_headers (http_stream) == -1)
+		if (stream_read (CAMEL_STREAM (http_stream), NULL, 0) == -1)
 			return NULL;
 	}
 
