@@ -3,7 +3,7 @@
  * Author:
  *  Michael Zucchi <notzed@ximian.com>
  *
- * Copyright 2000-2003 Ximian, Inc. (www.ximian.com)
+ * Copyright (C) 1999-2008 Novell, Inc. (www.novell.com)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of version 2 of the GNU Lesser General Public
@@ -33,10 +33,9 @@
 #include <glib/gstdio.h>
 
 #include <libedataserver/e-data-server-util.h>
-#include <libedataserver/e-memory.h>
-#include <libedataserver/e-msgport.h>
 
 #include "camel-file-utils.h"
+#include "camel-list-utils.h"
 #include "camel-object.h"
 
 #define d(x)
@@ -142,8 +141,8 @@ static GMutex *ref_lock;
 static GHashTable *type_table;
 
 /* fundamental types are accessed via global */
-CamelType camel_object_type;
-CamelType camel_interface_type;
+CamelType camel_object_type = CAMEL_INVALID_TYPE;
+CamelType camel_interface_type = CAMEL_INVALID_TYPE;
 
 #define P_LOCK(l) (pthread_mutex_lock(&l))
 #define P_UNLOCK(l) (pthread_mutex_unlock(&l))
@@ -1158,13 +1157,13 @@ camel_object_class_add_event(CamelObjectClass *klass, const char *name, CamelObj
 
 	pair = co_find_pair(klass, name);
 	if (pair) {
-		g_warning("camel_object_class_add_event: `%s' is already declared for '%s'",
+		g_warning("camel_object_class_add_event: '%s' is already declared for '%s'",
 			  name, klass->name);
 		return;
 	}
 
 	if (klass->magic == CAMEL_INTERFACE_MAGIC && prep != NULL) {
-		g_warning("camel_object_class_add_event: `%s', CamelInterface '%s' may not have an event prep function - ignored",
+		g_warning("camel_object_class_add_event: '%s', CamelInterface '%s' may not have an event prep function - ignored",
 			  name, klass->name);
 		prep = NULL;
 	}
@@ -1306,7 +1305,7 @@ camel_object_hook_event(void *vo, const char * name, CamelObjectEventHookFunc fu
 			}
 		}
 
-		g_warning("camel_object_hook_event: trying to hook event `%s' in class `%s' with no defined events.",
+		g_warning("camel_object_hook_event: trying to hook event '%s' in class '%s' with no defined events.",
 			  name, obj->klass->name);
 
 		return 0;
@@ -1343,7 +1342,7 @@ camel_object_remove_event(void *vo, unsigned int id)
 	g_return_if_fail (id != 0);
 
 	if (obj->hooks == NULL) {
-		g_warning("camel_object_unhook_event: trying to unhook `%u` from an instance of `%s' with no hooks",
+		g_warning("camel_object_unhook_event: trying to unhook '%u' from an instance of '%s' with no hooks",
 			  id, obj->klass->name);
 		return;
 	}
@@ -1373,7 +1372,7 @@ camel_object_remove_event(void *vo, unsigned int id)
 	}
 	camel_object_unget_hooks(obj);
 
-	g_warning("camel_object_unhook_event: cannot find hook id %u in instance of `%s'",
+	g_warning("camel_object_unhook_event: cannot find hook id %u in instance of '%s'",
 		  id, obj->klass->name);
 }
 
@@ -1389,7 +1388,7 @@ camel_object_unhook_event(void *vo, const char * name, CamelObjectEventHookFunc
 	g_return_if_fail (func != NULL);
 
 	if (obj->hooks == NULL) {
-		g_warning("camel_object_unhook_event: trying to unhook `%s` from an instance of `%s' with no hooks",
+		g_warning("camel_object_unhook_event: trying to unhook '%s' from an instance of '%s' with no hooks",
 			  name, obj->klass->name);
 		return;
 	}
@@ -1421,7 +1420,7 @@ camel_object_unhook_event(void *vo, const char * name, CamelObjectEventHookFunc
 	}
 	camel_object_unget_hooks(obj);
 
-	g_warning("camel_object_unhook_event: cannot find hook/data pair %p/%p in an instance of `%s' attached to `%s'",
+	g_warning("camel_object_unhook_event: cannot find hook/data pair %p/%p in an instance of '%s' attached to '%s'",
 		  (void *) func, data, obj->klass->name, name);
 }
 
@@ -1456,7 +1455,7 @@ camel_object_trigger_event(void *vo, const char * name, void *event_data)
 		}
 	}
 
-	g_warning("camel_object_trigger_event: trying to trigger unknown event `%s' in class `%s'",
+	g_warning("camel_object_trigger_event: trying to trigger unknown event '%s' in class '%s'",
 		  name, obj->klass->name);
 
 	return;
