@@ -26,18 +26,18 @@
 #define CAMEL_IMAP_STORE_H 1
 
 #include "camel-imap-types.h"
-#include <camel/camel-disco-store.h>
+#include <camel/camel-msgport.h>
+#include <camel/camel-offline-store.h>
 #include <sys/time.h>
 
 #ifdef ENABLE_THREADS
-#include <libedataserver/e-msgport.h>
 
 G_BEGIN_DECLS
 
 typedef struct _CamelImapMsg CamelImapMsg;
 
 struct _CamelImapMsg {
-	EMsg msg;
+	CamelMsg msg;
 
 	void (*receive)(CamelImapStore *store, struct _CamelImapMsg *m);
 	void (*free)(CamelImapStore *store, struct _CamelImapMsg *m);
@@ -60,7 +60,7 @@ G_END_DECLS
 G_BEGIN_DECLS
 
 enum {
-	CAMEL_IMAP_STORE_ARG_FIRST  = CAMEL_DISCO_STORE_ARG_FIRST + 100,
+	CAMEL_IMAP_STORE_ARG_FIRST  = CAMEL_OFFLINE_STORE_ARG_FIRST + 100,
 	CAMEL_IMAP_STORE_ARG_NAMESPACE,
 	CAMEL_IMAP_STORE_ARG_OVERRIDE_NAMESPACE,
 	CAMEL_IMAP_STORE_ARG_CHECK_ALL,
@@ -115,7 +115,7 @@ typedef enum {
 #define IMAP_FETCH_MINIMAL_HEADERS 3
 
 struct _CamelImapStore {
-	CamelDiscoStore parent_object;	
+	CamelOfflineStore parent_object;	
 	
 	CamelStream *istream;
 	CamelStream *ostream;
@@ -150,7 +150,7 @@ struct _CamelImapStore {
 };
 
 typedef struct {
-	CamelDiscoStoreClass parent_class;
+	CamelOfflineStoreClass parent_class;
 
 } CamelImapStoreClass;
 
